@@ -1,4 +1,5 @@
 import ctypes
+import locale
 import logging
 import os
 import sys
@@ -10,8 +11,12 @@ logger = logging.getLogger(__name__)
 
 TESSDATA_PREFIX = os.getenv('TESSDATA_PREFIX', None)
 libnames = []
+# 70 is the minimum credible dpi for tesseract and force it to compute an
+# estimate of the image dpi
+DPI_DEFAULT = 70
 
-if getattr(sys, 'frozen', False):
+
+if getattr(sys, 'frozen', False):  # pragma: no cover
     # Pyinstaller integration
     libnames += [os.path.join(sys._MEIPASS, "libtesseract-4.dll")]
     libnames += [os.path.join(sys._MEIPASS, "libtesseract-3.dll")]
@@ -26,7 +31,7 @@ if getattr(sys, 'frozen', False):
         TESSDATA_PREFIX = tessdata
 
 
-if sys.platform[:3] == "win":
+if sys.platform[:3] == "win":  # pragma: no cover
     libnames += [
         # Jflesch> Don't they have the equivalent of LD_LIBRARY_PATH on
         # Windows ?
@@ -51,12 +56,12 @@ else:
 g_libtesseract = None
 
 lib_load_errors = []
-for libname in libnames:
+for libname in libnames:  # pragma: no branch
     try:
         g_libtesseract = ctypes.cdll.LoadLibrary(libname)
         lib_load_errors = []
         break
-    except OSError as ex:
+    except OSError as ex:  # pragma: no cover
         if hasattr(ex, 'message'):
             # python 2
             lib_load_errors.append((libname, ex.message))
@@ -79,7 +84,8 @@ class PageSegMode(object):
     SINGLE_CHAR = 10
     SPARSE_TEXT = 11
     SPARSE_TEXT_OSD = 12
-    COUNT = 13
+    PSM_RAW_LINE = 13
+    COUNT = 14
 
 
 class Orientation(object):
@@ -128,7 +134,7 @@ class OSResults(ctypes.Structure):
     ]
 
 
-if g_libtesseract:
+if g_libtesseract:  # pragma: no cover
     g_libtesseract.TessVersion.argtypes = []
     g_libtesseract.TessVersion.restype = ctypes.c_char_p
 
@@ -342,12 +348,17 @@ if g_libtesseract:
 
 def init(lang=None):
     assert(g_libtesseract)
+
+    # Tesseract 4 workaround
+    if get_version() == "4.0.0":
+        locale.setlocale(locale.LC_ALL, "C")
+
     handle = g_libtesseract.TessBaseAPICreate()
     try:
         if lang:
             lang = lang.encode("utf-8")
         prefix = None
-        if TESSDATA_PREFIX:
+        if TESSDATA_PREFIX:  # pragma: no cover
             prefix = TESSDATA_PREFIX.encode("utf-8")
         g_libtesseract.TessBaseAPIInit3(
             ctypes.c_void_p(handle),
@@ -443,17 +454,18 @@ def set_image(handle, image):
     image.load()
     imgdata = image.tobytes("raw", "RGB")
 
-    imgsize = image.size
-
     g_libtesseract.TessBaseAPISetImage(
         ctypes.c_void_p(handle),
         imgdata,
-        ctypes.c_int(imgsize[0]),
-        ctypes.c_int(imgsize[1]),
+        ctypes.c_int(image.width),
+        ctypes.c_int(image.height),
         ctypes.c_int(3),  # RGB = 3 * 8
-        ctypes.c_int(imgsize[0] * 3)
+        ctypes.c_int(image.width * 3)
     )
 
+    dpi = image.info.get("dpi", [DPI_DEFAULT])[0]
+    g_libtesseract.TessBaseAPISetSourceResolution(handle, dpi)
+
 
 def recognize(handle):
     assert(g_libtesseract)
