@@ -8,7 +8,7 @@ import six
 
 def digits_only(string):
     """Return all digits that the given string starts with."""
-    match = re.match(r'(?P<digits>\d+)', string)
+    match = re.match(r'\D*(?P<digits>\d+)', string)
     if match:
         return int(match.group('digits'))
     return 0
@@ -17,7 +17,7 @@ def digits_only(string):
 def to_unicode(string):
     try:
         return six.u(string)
-    except:  # noqa: E722
+    except:  # noqa: E722  # pragma: no cover
         # probably already decoded
         return string
 
